"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Union
from typing_extensions import TypeAliasType


EventMetadataOutputTypedDict = TypeAliasType(
    "EventMetadataOutputTypedDict", Union[str, int, float, bool]
)


EventMetadataOutput = TypeAliasType("EventMetadataOutput", Union[str, int, float, bool])
