"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitMeterCreditPropertiesTypedDict(TypedDict):
    r"""Properties for a benefit of type `meter_unit`."""

    units: int
    rollover: bool
    meter_id: str


class BenefitMeterCreditProperties(BaseModel):
    r"""Properties for a benefit of type `meter_unit`."""

    units: int

    rollover: bool

    meter_id: str
