"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AggregationFunction(str, Enum):
    COUNT = "count"
    SUM = "sum"
    MAX = "max"
    MIN = "min"
    AVG = "avg"
    UNIQUE = "unique"
