"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerpaymentmethod import CustomerPaymentMethod, CustomerPaymentMethodTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerPaymentMethodCreateSucceededResponseTypedDict(TypedDict):
    payment_method: CustomerPaymentMethodTypedDict
    status: Literal["succeeded"]


class CustomerPaymentMethodCreateSucceededResponse(BaseModel):
    payment_method: CustomerPaymentMethod

    STATUS: Annotated[
        Annotated[Literal["succeeded"], AfterValidator(validate_const("succeeded"))],
        pydantic.Field(alias="status"),
    ] = "succeeded"
