"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionUpdateTrialTypedDict(TypedDict):
    trial_end: datetime
    r"""Set or extend the trial period of the subscription. If set to `now`, the trial will end immediately."""


class SubscriptionUpdateTrial(BaseModel):
    trial_end: datetime
    r"""Set or extend the trial period of the subscription. If set to `now`, the trial will end immediately."""
