"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerseat import CustomerSeat, CustomerSeatTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class SeatsListTypedDict(TypedDict):
    seats: List[CustomerSeatTypedDict]
    r"""List of seats"""
    available_seats: int
    r"""Number of available seats"""
    total_seats: int
    r"""Total number of seats for the subscription"""


class SeatsList(BaseModel):
    seats: List[CustomerSeat]
    r"""List of seats"""

    available_seats: int
    r"""Number of available seats"""

    total_seats: int
    r"""Total number of seats for the subscription"""
