"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .payment import Payment, PaymentTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceTypedDict(TypedDict):
    items: List[PaymentTypedDict]
    pagination: PaginationTypedDict


class ListResource(BaseModel):
    items: List[Payment]

    pagination: Pagination
