"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerPaymentMethodConfirmTypedDict(TypedDict):
    setup_intent_id: str
    set_default: bool


class CustomerPaymentMethodConfirm(BaseModel):
    setup_intent_id: str

    set_default: bool
