"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productcreateonetime import ProductCreateOneTime, ProductCreateOneTimeTypedDict
from .productcreaterecurring import (
    ProductCreateRecurring,
    ProductCreateRecurringTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ProductCreateTypedDict = TypeAliasType(
    "ProductCreateTypedDict",
    Union[ProductCreateOneTimeTypedDict, ProductCreateRecurringTypedDict],
)


ProductCreate = TypeAliasType(
    "ProductCreate", Union[ProductCreateOneTime, ProductCreateRecurring]
)
