"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattier import ProductPriceSeatTier, ProductPriceSeatTierTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ProductPriceSeatTiersTypedDict(TypedDict):
    r"""List of pricing tiers for seat-based pricing."""

    tiers: List[ProductPriceSeatTierTypedDict]
    r"""List of pricing tiers"""


class ProductPriceSeatTiers(BaseModel):
    r"""List of pricing tiers for seat-based pricing."""

    tiers: List[ProductPriceSeatTier]
    r"""List of pricing tiers"""
