"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerBalanceTypedDict(TypedDict):
    r"""Customer balance information."""

    balance: int
    r"""Customer balance in cents. Positive values represent credit (customer is owed money), negative values represent debit (customer owes money)."""
    currency: str
    r"""The currency code (ISO 4217) for the balance amount."""


class CustomerBalance(BaseModel):
    r"""Customer balance information."""

    balance: int
    r"""Customer balance in cents. Positive values represent credit (customer is owed money), negative values represent debit (customer owes money)."""

    currency: str
    r"""The currency code (ISO 4217) for the balance amount."""
