"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardpayment import CardPayment, CardPaymentTypedDict
from .genericpayment import GenericPayment, GenericPaymentTypedDict
from typing import Union
from typing_extensions import TypeAliasType


PaymentTypedDict = TypeAliasType(
    "PaymentTypedDict", Union[GenericPaymentTypedDict, CardPaymentTypedDict]
)


Payment = TypeAliasType("Payment", Union[GenericPayment, CardPayment])
