"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationCustomerEmailSettingsTypedDict(TypedDict):
    order_confirmation: bool
    subscription_cancellation: bool
    subscription_confirmation: bool
    subscription_cycled: bool
    subscription_past_due: bool
    subscription_revoked: bool
    subscription_uncanceled: bool
    subscription_updated: bool


class OrganizationCustomerEmailSettings(BaseModel):
    order_confirmation: bool

    subscription_cancellation: bool

    subscription_confirmation: bool

    subscription_cycled: bool

    subscription_past_due: bool

    subscription_revoked: bool

    subscription_uncanceled: bool

    subscription_updated: bool
